@@ -31,4 +31,15 @@
 #' @format data.table with columns DATE and GDP, quarterly frequency
 #' @source FRED
 #' @usage data(GDP)
-"GDP"
\ No newline at end of file
+"GDP"
+
+#' Stock & Watson US Dynamic Common Factor (1991) Data
+#' 
+#' @format data.table with columns date, ip (industrial production), 
+#' gmyxpg (personal income less transfer payments), 
+#' mtq (total manufacturing and trade sales), and 
+#' lpnag (employeess on nonagricultural payrolls)
+#' taken from 
+#' @source Kim and Nelson (1999) "State-Space Modesl with Regime Switching" (http://econ.korea.ac.kr/~cjkim/)
+#' @usage data(sw_dcf)
+"sw_dcf"
\ No newline at end of file
